# Copyright 2007 - 2015 Harri Pitkänen (hatapitk@iki.fi)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# This module contains general helper functions and classes for use
# with Python and Voikko.

import codecs
import os
import locale
import sys
import xml.dom.minidom
import gzip

# Word classes
NOUN=1
ADJECTIVE=2
VERB=3

# Vowel types
VOWEL_DEFAULT=0
VOWEL_FRONT=1
VOWEL_BACK=2
VOWEL_BOTH=3

# Gradation types
GRAD_NONE = 0
GRAD_SW = 1
GRAD_WS = 2

GRAD_WEAK = 3
GRAD_STRONG = 4

class FlagAttribute:
	"Vocabulary flag attribute"
	joukahainen = 0
	xmlGroup = None
	xmlFlag = None
	malagaFlag = None
	description = None

## Remove comments from a given line of text.
def removeComments(line):
	comment_start = line.find('#')
	if comment_start == -1:
		return line
	if comment_start == 0:
		return ''
	return line[:comment_start]

def readFlagAttributes(filename):
	"""Returns a map of flag attributes from given file. The keys in the
	map are in form xmlGroup/xmlFlag, such as 'compounding/ei_ys'."""
	inputfile = codecs.open(filename, 'r', 'UTF-8')
	flags = {}
	fileCont = True
	while fileCont:
		line = inputfile.readline()
		fileCont = line.endswith('\n')
		line = removeComments(line).strip()
		if len(line) > 0:
			f = FlagAttribute()
			endind = line.find(' ')
			f.joukahainen = int(line[:endind])
			line = line[endind:].strip()
			endind = line.find('/')
			f.xmlGroup = line[:endind]
			line = line[endind + 1:]
			endind = line.find(' ')
			f.xmlFlag = line[:endind]
			line = line[endind:].strip()
			endind = line.find(' ')
			if line[:endind] != '-': f.malagaFlag = line[:endind]
			line = line[endind:].strip()
			if len(line) > 0: f.description = line
			flags[f.xmlGroup + '/' + f.xmlFlag] = f
	inputfile.close()
	return flags

## Function that returns the type of vowels that are allowed in the suffixes for
# given simple word.
# The possible values are VOWEL_FRONT, VOWEL_BACK and VOWEL_BOTH.
def _simple_vowel_type(word):
	word = word.lower()
	last_back = max(word.rfind('a'), word.rfind('o'), word.rfind('å'), word.rfind('u'))
	last_ord_front = max(word.rfind('ä'), word.rfind('ö'))
	last_y = word.rfind('y')
	if last_back > -1 and max(last_ord_front, last_y) == -1:
		return VOWEL_BACK
	if last_back == -1 and max(last_ord_front, last_y) > -1:
		return VOWEL_FRONT
	if max(last_back, last_ord_front, last_y) == -1:
		return VOWEL_FRONT
	if last_y < max(last_back, last_ord_front):
		if last_back > last_ord_front: return VOWEL_BACK
		else: return VOWEL_FRONT
	else:
		return VOWEL_BOTH

## Returns autodetected vowel type of infection suffixes for a word.
# If word contains character '=', automatic detection is only performed on the
# trailing part. If word contains character '|', automatic detection is performed
# on the trailing part and the whole word, and the union of accepted vowel types is returned.
def get_wordform_infl_vowel_type(wordform):
	# Search for last '=' or '-', check the trailing part using recursion
	startind = max(wordform.rfind('='), wordform.rfind('-'))
	if startind == len(wordform) - 1: return VOWEL_BOTH # Not allowed
	if startind != -1: return get_wordform_infl_vowel_type(wordform[startind+1:])
	
	# Search for first '|', check the trailing part using recursion
	startind = wordform.find('|')
	if startind == len(wordform) - 1: return VOWEL_BOTH # Not allowed
	vtype_whole = _simple_vowel_type(wordform)
	if startind == -1: return vtype_whole
	vtype_part = get_wordform_infl_vowel_type(wordform[startind+1:])
	if vtype_whole == vtype_part: return vtype_whole
	else: return VOWEL_BOTH

def get_preference(prefname):
	'Returns the value of given preference'
	try:
		import voikko_dev_prefs
		if prefname == 'corevoikko' and hasattr(voikko_dev_prefs, 'corevoikko'):
			return voikko_dev_prefs.corevoikko
		if prefname == 'voikkotest_dir' and hasattr(voikko_dev_prefs, 'voikkotest_dir'):
			return voikko_dev_prefs.voikkotest_dir
		if prefname == 'voikkotest_build_options' and hasattr(voikko_dev_prefs, 'voikkotest_build_options'):
			return voikko_dev_prefs.voikkotest_build_options
		if prefname == 'voikko_data_dir' and hasattr(voikko_dev_prefs, 'voikko_data_dir'):
			return voikko_dev_prefs.voikko_data_dir
		if prefname == 'encoding' and hasattr(voikko_dev_prefs, 'encoding'):
			return voikko_dev_prefs.encoding
		if prefname == 'libvoikko_bin' and hasattr(voikko_dev_prefs, 'libvoikko_bin'):
			return voikko_dev_prefs.libvoikko_bin
		if prefname == 'diffviewcmd' and hasattr(voikko_dev_prefs, 'diffviewcmd'):
			return voikko_dev_prefs.diffviewcmd
	except ImportError:
		pass
	if prefname == 'corevoikko': return os.environ['HOME'] + '/git/corevoikko'
	if prefname == 'voikkotest_dir': return os.environ['HOME'] + '/tmp/voikkotest'
	if prefname == 'voikkotest_build_options': return ''
	if prefname == 'voikko_data_dir': return os.environ['HOME'] + '/git/corevoikko/data'
	if prefname == 'encoding': return locale.getpreferredencoding()
	if prefname == 'libvoikko_bin': return '/usr/bin'
	if prefname == 'diffviewcmd': return 'diff -U 0 "%s" "%s" | grep ^.C: 2>/dev/null | less'
	return None

## Returns True, if given character is a consonant, otherwise retuns False.
def is_consonant(letter):
	if letter.lower() in 'qwrtpsdfghjklzxcvbnm':
		return True
	else:
		return False

## Function that returns the type of vowels that are allowed in the affixes for given word.
# The possible values are VOWEL_FRONT, VOWEL_BACK and VOWEL_BOTH.
def vowel_type(word):
	word = word.lower()
	last_back = max(word.rfind('a'), word.rfind('o'), word.rfind('å'), word.rfind('u'))
	last_ord_front = max(word.rfind('ä'), word.rfind('ö'))
	last_y = word.rfind('y')
	if last_back > -1 and max(last_ord_front, last_y) == -1:
		return VOWEL_BACK
	if last_back == -1 and max(last_ord_front, last_y) > -1:
		return VOWEL_FRONT
	if max(last_back, last_ord_front, last_y) == -1:
		return VOWEL_FRONT
	if last_y < max(last_back, last_ord_front):
		if last_back > last_ord_front: return VOWEL_BACK
		else: return VOWEL_FRONT
	else:
		return VOWEL_BOTH


## Expands capital letters to useful character classes for regular expressions
def capital_char_regexp(pattern):
	pattern = pattern.replace('V', '(?:a|e|i|o|u|y|ä|ö|é|è|á|ó|â)')
	pattern = pattern.replace('C', '(?:b|c|d|f|g|h|j|k|l|m|n|p|q|r|s|t|v|w|x|z|š|ž)')
	pattern = pattern.replace('A', '(?:a|ä)')
	pattern = pattern.replace('O', '(?:o|ö)')
	pattern = pattern.replace('U', '(?:u|y)')
	return pattern

## Reads the word list in XML format specified by filename. If the name ends
# with .gz, the file is assumed to be gzip compressed. Calls function word_handler
# for each word, passing a XML Document object representing the word as a parameter.
# If show_progress == True, prints progess information to stdout
def process_wordlist(filename, word_handler, show_progress = False):
	if filename.endswith(".gz"):
		listfile = gzip.GzipFile(filename, 'r')
	else:
		listfile = open(filename, 'r', encoding="UTF-8")
	line = ""
	while line != '<wordlist xml:lang="fi">\n':
		line = listfile.readline()
		if line == '':
			sys.stderr.write("Malformed file " + filename + "\n")
			return
	
	wcount = 0
	while True:
		wordstr = ""
		line = listfile.readline()
		if line == "</wordlist>\n": break
		while line != '</word>\n':
			wordstr = wordstr + line
			line = listfile.readline()
		word = xml.dom.minidom.parseString(wordstr + line)
		word_handler(word.documentElement)
		wcount = wcount + 1
		if show_progress and wcount % 1000 == 0:
			sys.stdout.write("#")
			sys.stdout.flush()
	
	if show_progress: sys.stdout.write("\n")
	listfile.close()



# -*- coding: utf-8 -*-

# Copyright 2013-2018, 2020 Hannu Väisänen

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


# Program to generate old spellings and common spelling mistakes for Voikko lexicon.
# This automatic generation will generate some old
# spellings and spelling errors that do not exist in real life.

# Style- ja usage-lippujen arvot suoraan Joukahaisesta:
# grep -A1 '<style>' ../vocabulary/joukahainen.xml|grep flag|sort -u|gawk '{printf "%s,", substr($1,7,length($1)-13)}'
# grep -A1 '<usage>' ../vocabulary/joukahainen.xml|grep flag|sort -u|gawk '{printf "%s,", substr($1,7,length($1)-13)}'


import codecs
import getopt
import re
import sys
from types import *
sys.path.append("common")
import generate_lex_common

# Tulosta sanojen historiallinen taivutus.
# Mallia on otettu funktiosta handle_word(word) tiedostossa generate_lex.py.
#
import voikkoutils
import xml.dom.minidom
def print_historical_inflection (word):
    sukija_infclass = None
    for infclass in word.getElementsByTagName ("infclass"):
        if infclass.getAttribute("type") == "historical":
            sukija_infclass = generate_lex_common.tValue (infclass)
            break
    if sukija_infclass is not None:
        altforms = generate_lex_common.tValues (word.getElementsByTagName("forms")[0], "form")
        for altform in altforms:
            print ("SANA", altform, sukija_infclass)
#voikkoutils.process_wordlist (generate_lex_common.VOCABULARY_DATA + '/joukahainen.xml', \
#                              print_historical_inflection, False)
#sys.exit(1)


OPTIONS = generate_lex_common.get_options()

infile = codecs.open (OPTIONS["destdir"] + u"/all.lexc", "r", "UTF-8")
outfile = codecs.open (OPTIONS["destdir"] + u"/all-sukija.lexc", 'w', 'UTF-8')
sukijafile = codecs.open (OPTIONS["destdir"] + u"/poikkeavat-sukija.lexc", 'r', 'UTF-8')

C = "[qwrtpsšdfghjklzžxcvbnm]"      # Kerakkeet.
M = "[bcdfghklmnpqrsštvwxzži]"      # Kerakkeet - j + i.
J = "[bcdfghklmnpqrsštvwxzž]"       # Kerakkeet - j.
K = "[qwrtpsšdfghjklzžxcvbnmaiou]"  # Kerakkeet + ääntiöitä.
V = "[aeiouüyåäö]"                  # Ääntiöt.
A = "[aä]"
E = "[ei]"
O = "[oö]"
U = "[uy]"


def makeRePattern (wordClass, word):
    u = "^\\[%s\\](\\[I..\\])?\\[Xp\\].*%s\\[X\\]" % (wordClass, word)
#    outfile.write ("!===== " + u + " " + word + "\n")
    u = u.replace ('C', C)
    u = u.replace ('J', J)
    u = u.replace ('K', K)
    u = u.replace ('M', M)
    u = u.replace ('V', V)
    u = u.replace ('A', A)
    u = u.replace ('E', E)
    u = u.replace ('O', O)
    u = u.replace ('U', U)
#    outfile.write ("!===== " + u + "\n\n")
    return u


def makeRe (wordClass, word):
    return re.compile (makeRePattern (wordClass, word), re.UNICODE)


def replace (s, old, new):
    u = s.replace (old + u":",  new + u":")
    u = u.replace (old + u" ",  new + u" ")
    u = u.replace (old + u"\t", new + u"\t")
    u = u.replace (old + u"@",  new + u"@")
    return u


def replace_and_write (line, string1, string2):
    s = replace (line, string1, string2)
    outfile.write (s)

# Spor(a)adi, spor(a)adinen, jne.
#
re_aCi    = makeRe ("(Ln|Ln|Lnl|Le.)", "(.{2,}C)(a)([df])(i)")
re_aCia   = makeRe ("(Ln|Ln|Lnl|Le.)", "(.{2,}C)(a)([df]i)(a)")
re_aCinen = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{3,})(a)([df]i)(nen)")


# -omi, -oni, -ori jne -loppuiset sanat; myös -omia, -ominen, jne.
re_oMi    = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{1,}M)(o)([dfglmnr])(i)")
re_oMia   = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{1,}M)(o)([dfglmnr])(ia)")
re_oMinen = makeRe ("(Ll|Ln|Lnl|Le.)", "(.{1,}M)(o)([dfglmnr]i)(nen)")

re_aali    = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(aa)(l)(i)")
re_aalinen = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(aa)(li)(nen)")

re_iivi    = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(ii)(v)(i)")
re_iivinen = makeRe ("(Ll|Ln|Lnl)", "(.{2,})(ii)(vi)(nen)")

re_uuri = makeRe ("(Ll|Ln|Lnl)", "(.{1,}J)(uu)(r)(i)") # Ei muuteta juuri-loppuisia yhdyssanoja.
re_yyri = makeRe ("(Ll|Ln|Lnl)", "(.{1,}C)(yy)(r)(i)")
re_ööri = makeRe ("(Ll|Ln|Lnl)", "(.{1,}C)(öö)(r)(i)")

re_torio = makeRe ("(Ll|Ln|Lnl)", "(.{1,}t)(o)(rio)")

def find (line, list):
    for u in list:
#        outfile.write ("!!!! " + u + " " + list)
        if line.find(u) >= 0:
            return True
    return False


STOP = re.compile ("( |\t|@)")

def end (line):
    n = line.find(":")
    r = STOP.search(line[n+1:])
    return (n, n+r.start(1)+2)


def ga (line, x):
    if not find (line, x[0]):
        e = end (line)
        p = line[e[0]:e[1]]
        q = p[:-x[1]-1] + x[2] + p[-x[3]-1:]
#        if line.find("bromi") > 0:
#            outfile.write ("!!HUUHAA [" + p + "]  [" + q + "]\n")
        outfile.write (line.replace (p, q))


re_oittaa1 = makeRe (u"Lt", u".Koittaa")
re_oittaa2 = makeRe (u"Lt", u".Köittää")

re_ottaa1 = makeRe (u"Lt", u".Kottaa")
re_ottaa2 = makeRe (u"Lt", u".Köttää")

re_oitella1 = makeRe (u"Lt", u".Koitella")
re_oitella2 = makeRe (u"Lt", u".Köitellä")

re_otella1 = makeRe (u"Lt", u".Kotella")
re_otella2 = makeRe (u"Lt", u".Kötellä")

re_ottua1 = makeRe (u"Lt", u".Kottua")
re_ottua2 = makeRe (u"Lt", u".Köttyä")

re_oittua1 = makeRe (u"Lt", u".Koittua")
re_oittua2 = makeRe (u"Lt", u".Köittyä")

re_isoida = makeRe (u"Lt", u"isoida") # Organisoida => organiseerata.

re_nuolaista = re.compile (u"\\[Lt\\].* Nuolaista_", re.UNICODE)
re_rangaista = re.compile (u"\\[Lt\\].* Rangaista_", re.UNICODE)

re_Xiljoona = re.compile (u"\\A(?:\\[Bc\\]|\\[Sn\\]|@).*(b|m|tr)iljoon", re.UNICODE)

re_isoida_x = re.compile (u"\\A\[Lt\]\[Xp\](dramatisoida|karakterisoida)\[X\]")

re_A = re.compile (u"[aou]")

re_oitin = makeRe (u"Ln", u".Coitin")
re_oite  = makeRe (u"Ln", u".Coite")

re_aatio = makeRe (u"Ln", u"..aatio")
re_uutio = makeRe (u"Ln", u".Cuutio")
re_tio   = makeRe (u"Ln", u"([^a]i|k|p)tio") # Traditio, funktio, mutta ei aitio.
re_uusio = makeRe (u"Ln", u"Cuusio")         # Myös fuusio.

re_mmainen = re.compile (".*mmainen\\[X\\]")
re_mmäinen = re.compile (".*mmäinen\\[X\\]")


def g2 (line, x):
    outfile.write (line.replace (x[0], x[1]))


def g4 (line, x):
    s = line.replace (x[0], x[1])
    outfile.write (s.replace (x[2], x[3]))


def g6 (line, x):
    if line.find(x[4]) > 0:
        s = line.replace (x[4], x[5])
        s = s.replace (x[0], x[1])
        s = s.replace (x[2], x[3])
        outfile.write (s)


def replace4 (s, old, new, u):
    return s.replace (old + u, new + u)


def replace5 (s, old, new, u, v):
    p = replace4 (s, old, new, u)
    return replace4 (p, old, new, v)


def gn (line, x):
    if not line.startswith("[Ln][Xp]kuusio[X]"):
        for u in x[1]:
            s = replace5 (line, x[0], u, " ", "\t")
            outfile.write (s)
        s = line.replace (x[0] + "[X]", x[2][0] + "i[X]")
        s = s.replace (x[3][0], x[3][1])
        s = replace (s, x[0], x[2][0])
        outfile.write (s)
        s = replace5 (s, x[2][0], x[2][1], " ", "\t")
        outfile.write (s)
        s = line.replace (x[3][0], x[3][1])
        replace_and_write (s, x[0], x[2][0])
        replace_and_write (s, x[0], x[2][1])


def g10 (line, x):
    if not x[8].match(line):
        s = line.replace(x[0],x[1]).replace(x[4],x[5]).replace(x[6],x[7])
        replace_and_write (s, x[2], x[3])


list_oMi    = ["blogi", "kromi", "aasiori", "hevosori", "jalostusori", "siitosori"]
list_oMia   = ["victoria", "viktoria"]
list_oMinen = ["hiusten=halkominen"]
list_aali   = ["kraali", "shaali"]

list_all = [
    (re_nuolaista, g2, ("Nuolaista_", "SukijaNuolaista_")),
    (re_rangaista, g2, ("Rangaista_", "SukijaRangaista_")),
    (re_isoida,   g10, ("isoida", "iseerata", "iso", "iseer", "Kanavoida", "Saneerata", "Voida", "Saneerata", re_isoida_x)),

    (re_aCi,       ga, ([], 1, "a", 1)),
    (re_aCia,      ga, ([], 2, "a", 2)),
    (re_aCinen,    ga, ([], 2, "a", 2)),

    (re_oMi,       ga, (list_oMi,    1, "o", 1)),
    (re_oMia,      ga, (list_oMia,   2, "o", 2)),
    (re_oMinen,    ga, (list_oMinen, 2, "o", 2)),

#    (re_aali,      ga, (list_aali, 2, "", 1)),
#    (re_aalinen,   ga, ([],        3, "", 2)),

#    (re_iivi,      ga, (["aktiivi"], 2, "", 1)),
#    (re_iivinen,   ga, ([], 3, "", 2)),

#    (re_uuri,      ga, (["kulttuuri"], 2, "", 1)),
#    (re_yyri,      ga, ([], 2, "", 1)),
#    (re_ööri,      ga, ([], 2, "", 1)),

#    (re_torio,     ga, ([], 3, "o", 3)),

    (re_oitin,     g6, ("oit:", "oit:", "oit ", "ot ",  "Suodatin",   "Suodatin")),
    (re_oite,      g6, ("oit:", "oit:", "oit ", "ot ",  "Vaate",      "Vaate")),
    (re_oittaa1,   g6, ("o:",   "oit:", "o ",   "ot ",  "Kirjoittaa", "Alittaa")),
    (re_oittaa2,   g6, ("ö:",   "öit:", "ö ",   "öt ",  "Kirjoittaa", "Alittaa")),
    (re_oittaa1,   g6, ("oit:", "oit:", "oit ", "ot ",  "Alittaa",    "Alittaa")),
    (re_oittaa2,   g6, ("öit:", "öit:", "öit ", "öt ",  "Alittaa",    "Alittaa")),
    (re_ottaa1,    g6, ("ot:",  "ot:",  "ot ",  "oit ", "Alittaa",    "Alittaa")),
    (re_ottaa2,    g6, ("öt:",  "öt:",  "öt ",  "öit ", "Alittaa",    "Alittaa")),
    (re_ottaa1,    g6, ("o:",   "o:",   "o ",   "oi ",  "Ammottaa",   "Ammottaa")),
    (re_ottaa2,    g6, ("ö:",   "ö:",   "ö ",   "öi ",  "Ammottaa",   "Ammottaa")),
    (re_oitella1,  g6, ("oit:", "oit:", "oit ", "ot ",  "Aatella",    "Aatella")),
    (re_oitella2,  g6, ("öit:", "öit:", "öit ", "öt ",  "Aatella",    "Aatella")),
    (re_otella1,   g6, ("ot:",  "ot:",  "ot ",  "oit ", "Aatella",    "Aatella")),
    (re_otella2,   g6, ("öt:",  "öt:",  "öt ",  "öit ", "Aatella",    "Aatella")),
    (re_ottua1,    g6, ("ot:",  "ot:",  "ot ",  "oit ", "Asettua",    "Asettua")),
    (re_ottua2,    g6, ("öt:",  "öt:",  "öt ",  "öit ", "Asettua",    "Asettua")),
    (re_oittua1,   g6, ("oit:", "oit:", "oit ", "ot ",  "Asettua",    "Asettua")),
    (re_oittua2,   g6, ("öit:", "öit:", "öit ", "öt ",  "Asettua",    "Asettua")),

    (re_uusio,     gn, ("uusio", ("usio", ),        ("usion",  "usioon"),  ("NimisanaAutio_a", "NimisanaPaperi_a"))),
    (re_tio,       gn, ("tio",   ("tsio", ),        ("tsion",  "tsioon"),  ("NimisanaAutio_a", "NimisanaPaperi_a"))),
    (re_aatio,     gn, ("aatio", ("atio", "atsio"), ("atsion", "atsioon"), ("NimisanaAutio_a", "NimisanaPaperi_a"))),
    (re_uutio,     gn, ("uutio", ("utio", "utsio"), ("utsion", "utsioon"), ("NimisanaAutio_a", "NimisanaPaperi_a"))),

    (re_mmainen,   g4, ("mmai ", "mai ", "mmai\t", "mai ")),
    (re_mmäinen,   g4, ("mmäi ", "mäi ", "mmäi\t", "mäi ")),
]


def generate_all (line, pattern_list):
    for x in pattern_list:
        if x[0].match(line):
            x[1] (line, x[2])
        


def word_class (line):
    L = dict ([("[Lep]", "Paikannimi"),
               ("[Ll]",  "Laatusana"),
               ("[Ln]",  "Nimisana"),
               ("[Lnl]", "NimiLaatusana"),
               ("[Lt]",  "")])
    return L[line[0:line.find("]")+1]]


# Sanoja, joilla on vain muutama vanha taivutusmuoto. Generoidaan ne erikseen,
# mutta vain sanoille, jotka ovat Joukahaisessa. Sanat ovat Nykysuomen
# sanakirjan taivutuskaavojen numeroiden mukaisessa järjestyksessä.
#
# Tuomo Tuomi: Suomen kielen käänteissanakirja, 2. painos.
# Suomalaisen Kirjallisuuden Seura 1980.

def write_word (line, word, lexicon):
    prefix = line[0:line.find (u" ")]
    A = u"a" if re_A.search(word) else u"ä"
    outfile.write (u"%s %s%s_%s ;\n" % (prefix, word_class(line), lexicon, A))

def write_ahven (line, word):
    if not line.startswith (u"[Lu]"):
        write_word (line, word, u"SukijaAhven")

def write_virkkaa (line, word):
    prefix = line[0:line.find (u" ")]
    outfile.write (u"%s SukijaVirkkaa_ä ;\n" % (prefix))

def write_paistaa (line, word):
    prefix = line[0:line.find (u" ")]
    outfile.write (u"%s SukijaPaistaa_a ;\n" % (prefix))

def write_paahtaa (line, word):
    prefix = line[0:line.find (u" ")]
    outfile.write (u"%s SukijaPaahtaa_a ;\n" % (prefix))

def write_taittaa (line, word):
    prefix = line[0:line.find (u" ")]
    outfile.write (u"%s SukijaTaittaa_a ;\n" % (prefix))

def write_palata (line, word):
    prefix = line[0:line.find (u" ")]
    outfile.write (u"%s SukijaPalata_a ;\n" % (prefix))


word_list = [
    (u"tällainen",       ((u"tällai",      u"tällai", u"NimiLaatusanaNainenInen_a", u"NimiLaatusanaNainenInen_ä"),
                          (u"tällai",      u"tälläi", u"NimiLaatusanaNainenInen_a", u"NimiLaatusanaNainenInen_aä"))),

    (u"lainen",  lambda line, word: replace_and_write (line.replace(u"lai",u"läi"), u"NimiLaatusanaNainen_a", u"NimiLaatusanaNainen_ä")),

    # 38 pieni (4, 4). Juoni, moni, pieni, tyyni, (peilityyni, rasvatyyni).
    #
    # Nämä ovat tiedostossa poikkeavat-sukija.lexc
    #
##    (u"juoni", [u"[Lnl][Xp]juoni[X]juon:juon NimiLaatusanaSukijaPieni_a ;"]),
##    (u"moni",  [u"[Ln][Xp]moni[X]mon:mon NimisanaSukijaPieni_a ;"]),
##    (u"pieni", [u"[Ll][Xp]pieni[X]pien:pien LaatusanaSukijaPieni_ä ;"]),
##    (u"tyyni", [u"[Ll][Xp]tyyni[X]tyyn:tyyn LaatusanaSukijaPieni_ä ;"]),
##    (u"peilityyni", [u"[Ll][Xp]peilityyni[X]peili[Bm]tyyn:peilityyn LaatusanaSukijaPieni_ä ;"]),
##    (u"rasvatyyni", [u"[Ll][Xp]rasvatyyni[X]rasva[Bm]tyyn:rasvatyyn LaatusanaSukijaPieni_ä ;"]),

    # 39 nuori (3, 3). Tuomi, s. 182, 184.
    #
    # Nämä ovat tiedostossa poikkeavat-sukija.lexc
    #
#    (u"juuri",   [u"[Ln][Xp]juuri[X]juur[Ses][Ny]na:juurna NimisanaLiOlV_a ;",
#                  u"[Ln][Xp]juuri[X]juur[Ses][Ny]ra:juurra NimisanaLiOlV_a ;"]),
#    (u"nuori",   [u"[Lnl][Xp]nuori[X]nuor[Ses][Ny]na:nuorna NimisanaLiOlV_a ;",
#                  u"[Lnl][Xp]nuori[X]nuor[Ses][Ny]ra:nuorra NimisanaLiOlV_a ;"]),
#    (u"suuri",   [u"[Lnl][Xp]suuri[X]suur[Ses][Ny]na:suurna NimisanaLiOlV_a ;",
#                  u"[Lnl][Xp]suuri[X]suur[Ses][Ny]ra:suurra NimisanaLiOlV_a ;"]),

    # 46 hapsi (1, 1). Tuomi, s. 190. -- Vvfst tunnistaa muodot "hasten" ja "hapsien".
    # hasna, hassa, hasten, hapsien   -- Nämä ovat niin harvinaisia, että tarvitseeko näitä indeksoinnissa?
    #
#    (u"hapsi", [u"[Ln][Xp]hapsi[X]has[Ses][Ny]na:hasna NimisanaLiOlV_a ;",
#                u"[Ln][Xp]hapsi[X]has[Ses][Ny]sa:hassa NimisanaLiOlV_a ;"]),

    # 79 terve (4, 4). Tuomi s. 142, 143, 146.
    #
    # Nämä ovat tiedostossa poikkeavat-sukija.lexc
    #
#    (u"tuore", [u"[Ll][Xp]tuore[X]tuore[Ses][Ny]nna:tuorenna NimisanaLiOlV_a ;"]),
#    (u"vetre", [u"[Ll][Xp]vetre[X]vetre[Ses][Ny]nnä:vetrennä NimisanaLiOlV_ä ;"]),
#    (u"päre",  [u"[Ln][Xp]päre[X]päre[Ses][Ny]nnä:pärennä NimisanaLiOlV_ä ;"]),
#    (u"terve", [u"[Lnl][Xp]terve[X]terve[Ses][Ny]nnä:tervennä NimisanaLiOlV_ä ;"]),

##    (u"kaivu", [u"[Ln][Xp]kaivu[X]kaivu[Sill][Ny]usee:kaivuusee NimisanaLiOlN_a ;"]),
#    (u"kaivu", [u"[Ln][Xp]kaivu[X]kaivu:kaivu NimisanaPuu_a ;",
#                u"[Ln][Xp]kaivu[X]kaivu[Sill][Ny]usee:kaivuusee NimisanaLiOlN_a ;"]),
]


function_list = [
    # Herttua-tyyppisillä sanoilla on monikkomuodot, joissa ei ole o:ta (herttuilla, jne).
    #
    # 20 herttua (10, 10). Tuomi, s. 114, 116, 121, 124, 125.
    #
    (lambda line, word: outfile.write (u"[Ln][Xp]%s[X]%s:%s SukijaHerttua ;\n" %
                                       (word, word[0:len(word)-1], word[0:len(word)-1])),
     (u"aurtua",
      u"herttua",
      u"hierua",
      u"juolua",
      u"lastua",
      u"liettua",
      u"luusua",
      u"porstua",
      u"saarua",
      u"tanhua")),

# Vapaa ja tienoo ovat taivutuskaavoina SukijaVapaa.
#
    # 23 vapaa (8, 8). Tuomi, s. 1, 2.
    #
#    (write_vapaa_tienoo,
#     (u"kajaa",
#      u"vajaa",
#      u"vakaa",
#      u"suklaa",
#      u"harmaa",
#      u"vapaa",
#      u"nepaa",
#      u"hurraa")),

    # 24 tienoo (14, 14). Tuomi, s. 345. Taipuu kuten vapaa.
    #
#    (write_vapaa_tienoo,
#     (u"kabeljoo",
#      u"kalikoo",
#      u"pikoo",
#      u"talkoo",
#      u"haloo",
#      u"halloo",
#      u"tienoo",
#      u"poppoo",
#      u"bigarroo",
#      u"platoo",
#      u"ehtoo",
#      u"palttoo",
#      u"ponttoo",
#      u"nivoo")),

    # 33 lohi (2, 2). Tuomi, s. 151.
    # lohten, uuhten
    #
    (lambda line, word: outfile.write (u"[Ln][Xp]%s[X]%s:%s SukijaLohi ;\n" %
                                       (word, word[0:len(word)-1], word[0:len(word)-1])),
     (u"lohi",
      u"tyynen=meren=lohi",   # On Joukahaisessa.
      u"uuhi")),

    # 34 lahti (2, 2). Tuomi, s. 193.
    # lahta (= lahtea), lahtein
    #
    (lambda line, word: write_word (line, word, "SukijaLahti"),
     (u"haahti",
      u"lahti")),

    # Ahven taipuu kuten sisar, paitsi että yksikön olento on myös ahvenna.
    #
    # 55 ahven (22, 23). Tuomi, s. 246, 247, 301, 302.
    #
    (write_ahven,
     (u"aamen",
      u"ahven",
      u"haiven",
      u"huomen",
      u"häiven",
      u"höyhen",
#      u"ien",  # On erikseen: ikene, ien.
      u"iljen",
      u"joutsen",
      u"jäsen",
      u"kymmen",
      u"kämmen",
      u"liemen",
      u"paimen",
      u"siemen",
      u"ruumen",
      u"terhen",
      u"taimen",
      u"tuumen",
      u"tyven",
      u"tyyven",
      u"uumen",
      u"vuomen")),

    # 69 kaunis (7, 6). Tuomi, s. 358.
    #
     (lambda line, word: write_word (line, word, "SukijaKaunis"),
     (u"kallis",
      u"aulis",
      u"valmis",
      u"kaunis",
#      u"altis",
      u"tiivis")),

    (lambda line, word: write_word (line, word, "SukijaAltis"), ("altis", )),

     # 11 paistaa (9, 9). Tuomi s. 1, 2, 8, 11, 12, 15, 17.
     #
     (write_virkkaa, ("vilkkaa",
                      "virkkaa")),
     (write_paistaa, ("maustaa",  # Ei ole Tuomella, mutta Lönnrot-projektin kirjoissa taivutetaan maustin, maustit, maustimme...
                      "paistaa",
                      "raistaa",
                      "uppo=paistaa")),
     (write_paahtaa, ("kaihtaa",
                      "paahtaa", )),
     (write_taittaa, ("naittaa", # Onko Tuomen kirjassa?
                      "saattaa",
                      "taittaa",
                      "palttaa",
                      "varttaa")),

     # Avata -> avaja jne. Nykysuomen sanakirjan teonsanojen taivutuskaava 40.
     (write_palata,  ("avata",
                      "halata",
                      "palata")),
     (lambda line, word: write_word (line, word, "SukijaLevätä"), ("levätä", )),
     (lambda line, word: write_word (line, word, "SukijaHylätä"), ("hyljätä", "hylätä")),
]

def convert_to_dictionary (word_list):
    l0 = map (lambda x : x[0], word_list)
    l1 = map (lambda x : x[1], word_list)
    return dict (zip (l0, l1))

sukija_dictionary = convert_to_dictionary (word_list)


def error (line):
    sys.stderr.write (line)
    sys.stderr.write ("Wrong format in sukija_dictionary.\n")
    sys.exit (1)


def write_list (line, key, data):
    for x in data:
        if type(x) == str:
            outfile.write (x + u"\n")
        else:
            error (line)


def write_tuple (line, key, g):
     if type(g[0]) == str:
         for i in range (1, len(g)):
             replace_and_write (line, g[0], g[i])
     elif type(g[0]) == tuple:
         for i in range (0, len(g)):
             if (len(g[i]) == 2):
                 replace_and_write (line, g[i][0], g[i][1])
             else:
                 s = line.replace (g[i][2], g[i][3])
                 outfile.write (replace (s, g[i][0], g[i][1]))
     else:
         error (line)


# Extract base form from a line.
#
base_form_re = re.compile (u"\\[Xp\\]([^[]+)\\[X\\]", re.UNICODE)

def generate_word (r, line, sukija_dictionary):
    try:
        g = sukija_dictionary[r.group(1)]
        if type(g) == list:
            write_list (line, r.group(1), g)
        elif type(g) == tuple:
            write_tuple (line, r.group(1), g)
        elif type(g) == LambdaType:
            g (line, r.group(1))
        else:
            error (line)
    except KeyError:  # It is not an error if a word is not in sukija_dictionary.
        pass


def generate_from_function (r, line, function_list):
    for x in function_list:
        if r.group(1) in x[1]:
            x[0] (line, r.group(1))


def generate_xiljoona (line):
    if (line.startswith ("[Sn]")):
        u = line.replace (u"miljoona@", u"miljona@")
        u = u.replace (u"miljoonat@", u"miljonat@")
        u = u.replace (u"biljoona@",  u"biljona@")
        u = u.replace (u"biljoonat@", u"biljonat@")
        u = u.replace (u"triljoona@",  u"triljona@")
        u = u.replace (u"triljoonat@", u"triljonat@")
        outfile.write (u)
    else:
        outfile.write (line.replace (u"iljoon", u"iljon"))
        if (line.startswith (u"@") and line.find (u"iljoonien:")):
            outfile.write (line.replace (u"iljoonien", u"iljoonain"))
            outfile.write (line.replace (u"iljoonien", u"iljonain"))


def write_arvaella (line):
    if line.startswith("[Lt][Xp]") and line.find ("Arvailla_") > 0:
        if not line.startswith ("[Lt][Xp]piillä[X]") and not line.startswith ("[Lt][Xp]viillä[X]"):
            line = line.replace ("il:", ":").replace("il ", " ").replace ("Arvailla_", "SukijaArvaella_")
            outfile.write (line)


ei_vertm = re.compile (u"@[PDC][.]EI_VERTM([.]ON)?@", re.UNICODE)
ei_yks = "@P.EI_YKS.ON@"


# Yksikintoista, yksikääntoista, kaksikaantoista, jne.
#
LUKUSANA_TOISTA =\
  "[Bc]toista[Xp][X]:kintoista	Liitesana_a	;\n" +\
  "[Bc]toista:kintoista	LukusananJälkiliite	;\n" +\
  "[Bc]toista:kintoista	LukusanaKerroinTuhatta	;\n" +\
  "[Bc]toista:kintoista	LukusananAlkuKerroin	;\n" +\
  "[Bc]toista[Xp][X]:kaantoista	Liitesana_a	;\n" +\
  "[Bc]toista:kaantoista	LukusananJälkiliite	;\n" +\
  "[Bc]toista:kaantoista	LukusanaKerroinTuhatta	;\n" +\
  "[Bc]toista:kaantoista	LukusananAlkuKerroin	;\n" +\
  "[Bc]toista[Xp][X]:kääntoista	Liitesana_a	;\n" +\
  "[Bc]toista:kääntoista	LukusananJälkiliite	;\n" +\
  "[Bc]toista:kääntoista	LukusanaKerroinTuhatta	;\n" +\
  "[Bc]toista:kääntoista	LukusananAlkuKerroin	;\n"


# Yhdeskintoista jne.
#
JÄRJESTYSLUKU_TOISTA = \
  "[Bc]toista[Xp][X]:kintoista	Liitesana_a	;\n" +\
  "[Bc]toista:kintoista	LukusananJälkiliiteJl	;\n" +\
  "[Bc]toista[Xp][X]:kaantoista	Liitesana_a	;\n" +\
  "[Bc]toista:kaantoista	LukusananJälkiliiteJl	;\n" +\
  "[Bc]toista[Xp][X]:kääntoista	Liitesana_a	;\n" +\
  "[Bc]toista:kääntoista	LukusananJälkiliiteJl	;\n"


sukija_additions = {
    "LEXICON Asemosana\n":     "SukijaAsemosana ;\n",
    "LEXICON Sanasto_ee\n":    "SukijaPoikkeavat_ee ;\n",
    "LEXICON Sanasto_em\n":    "SukijaPoikkeavat_em ;\n",
    "LEXICON Sanasto_ep\n":    "SukijaPoikkeavat_ep ;\n",
    "LEXICON Sanasto_h\n":     "SukijaPoikkeavat_h ;\n",
    "LEXICON Sanasto_l\n":     "SukijaPoikkeavat_l ;\n",
    "LEXICON Sanasto_n\n":     "SukijaPoikkeavat_n ;\n",
    "LEXICON Sanasto_nl\n":    "SukijaPoikkeavat_nl ;\n",
    "LEXICON Sanasto_p\n":     "Sukija_p ;\n",
    "LEXICON Sanasto_t\n":     "SukijaPoikkeavat_t ;\n",
    "LEXICON Sanasto_s\n":     "SukijaPoikkeavat_s ;\n",
    "LEXICON Suhdesana\n":     "SukijaSuhdesana ;\n",
    "LEXICON LukusananErikoisjälkiliite\n": "SukijaLukusananErikoisjälkiliite ;\n",
    "LEXICON Lukusana\n":            "SukijaLukusana ;\n",
    "LEXICON Järjestysluku\n":       "SukijaJärjestysluku ;\n",
    "LEXICON LukusanaToista\n":      LUKUSANA_TOISTA, 
    "LEXICON JärjestyslukuToista\n": JÄRJESTYSLUKU_TOISTA,
    "LEXICON Omistusliite_a\n":  "[O2y]s:s Liitesana_a  ;\n",
    "LEXICON Omistusliite_ä\n":  "[O2y]s:s Liitesana_ä  ;\n",
    "LEXICON Omistusliite_aä\n": "[O2y]s:s Liitesana_aä ;\n"
#   "LEXICON Omistusliite_a\n":  "[O1y]in:in Liitesana_a  ;\n[O2y]s:s Liitesana_a  ;\n",
#   "LEXICON Omistusliite_ä\n":  "[O1y]in:in Liitesana_ä  ;\n[O2y]s:s Liitesana_ä  ;\n",
#   "LEXICON Omistusliite_aä\n": "[O1y]in:in Liitesana_aä ;\n[O2y]s:s Liitesana_aä ;\n"
}


def write_sukija_additions (line, sukija_additions):
    try:
        outfile.write (sukija_additions[line])
    except KeyError:
        pass


# Copy Voikko vocabulary and insert forms that Sukija needs.
#
while True:
    line = infile.readline()
    if line == u"":
        break
    if line.find (u"[Tn4]mi@") == 0:  # 4. nimitapa (puhu+minen) ei ole teonsanan taivutusmuoto.
        continue
    if line.find ("vihanta[X]") >= 0:
        line = line.replace ("Emäntä_", "SukijaVihanta_")
    line = re.sub (ei_vertm, "", line)
    if line.find (ei_yks) > 0 and line.startswith ("[L"):
        if not line.startswith ("[Ln][Xp]lehdes[X]"):
            line = line.replace (ei_yks, "")
    if line.find (u"=") >= 0:
        line = line.replace (u"@P.YS_EI_JATKOA.ON@", u"")
    if line.find ("lähtöinen") >= 0:
        line = line.replace ("@R.YS_ALKANUT@", "")

    if OPTIONS["sukija-ys"]:
        line = line.replace (u"@P.YS_EI_JATKOA.ON@", u"")
        line = line.replace (u"@D.YS_EI_JATKOA@", u"")
        line = line.replace (u"@C.YS_EI_JATKOA@", u"")

    outfile.write (line)
    write_sukija_additions (line, sukija_additions)
    generate_all (line, list_all)
    write_arvaella (line)

    r = base_form_re.search (line)
    if r:
        generate_word (r, line, sukija_dictionary)
        generate_from_function (r, line, function_list)
    if (re_Xiljoona.search (line)):
        generate_xiljoona (line)
infile.close()

outfile.write (u"\n\n\n")

while True:
    line = sukijafile.readline()
    if line == u"":
        break
    if u"!" in line:
        line = line[0:line.find(u"!")]
    outfile.write (line)
sukijafile.close()

outfile.close()



# Copyright 2012 - 2015 Harri Pitkänen (hatapitk@iki.fi)
# Program to generate lexicon files for voikko-fi

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
sys.path.append("common")
import hfconv
import generate_lex_common
import voikkoutils
import codecs


def addDiacritics(line):
	if "[Nm]" in line or ("[Sn][Ny]" in line and "Omistusliite" in line):
		middle = line.find(":")
		return "@C.EI_YKS@" + line[0:middle+1] + "@C.EI_YKS@" + line[middle+1:]
	return line

def replacementsFront(line):
	return addDiacritics(line.replace("<A>", "ä").replace("<O>", "ö").replace("<U>", "y"))


def replacementsBack(line):
	return addDiacritics(line.replace("<A>", "a").replace("<O>", "o").replace("<U>", "u"))

def filterLines(lines, lexiconPrefix):
	for line in lines:
		if line.startswith("?Laatusana"):
			if lexiconPrefix in ["Laatusana", "NimiLaatusana"]:
				yield line[10:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?NimiLaatusana"):
			if lexiconPrefix == "NimiLaatusana":
				yield line[14:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?NimisanaOnly"):
			if lexiconPrefix == "Nimisana":
				yield line[13:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Nimisana"):
			if lexiconPrefix in ["Nimisana", "NimiLaatusana"]:
				yield line[9:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?NotLaatusana"):
			if lexiconPrefix not in ["Laatusana", "Asemosana"]:
				yield line[13:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Paikannimi"):
			if lexiconPrefix == "Paikannimi":
				yield line[11:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Sukunimi"):
			if lexiconPrefix == "Sukunimi":
				yield line[9:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Etunimi"):
			if lexiconPrefix == "Etunimi":
				yield line[8:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Nimi"):
			if lexiconPrefix == "Nimi":
				yield line[5:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Erisnimi"):
			if lexiconPrefix in ["Sukunimi", "Etunimi", "Paikannimi", "Nimi"]:
				yield line[9:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Yleisnimi"):
			if lexiconPrefix in ["Nimisana", "Laatusana", "NimiLaatusana"]:
				yield line[10:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?ErisYleisOnly"):
			if lexiconPrefix in ["Nimisana", "Sukunimi", "Etunimi", "Paikannimi", "Nimi"]:
				yield line[14:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?Asemosana"):
			if lexiconPrefix == "Asemosana":
				yield line[10:].replace("<WC>", lexiconPrefix)
		elif line.startswith("?NotAsemosana"):
			if lexiconPrefix != "Asemosana":
				yield line[13:].replace("<WC>", lexiconPrefix)
		else:
			yield line.replace("<WC>", lexiconPrefix)

def appendLines(lexiconPrefix, lexiconName, lines, lexcFile):
	lexcFile.write("LEXICON " + lexiconPrefix + lexiconName + "_a\n")
	for line in filterLines(lines, lexiconPrefix):
		lexcFile.write(replacementsBack(line) + "\n")
	lexcFile.write("LEXICON " + lexiconPrefix + lexiconName + "_ä\n")
	for line in filterLines(lines, lexiconPrefix):
		lexcFile.write(replacementsFront(line) + "\n")
	lexcFile.write("LEXICON " + lexiconPrefix + lexiconName + "_aä\n")
	for line in filterLines(lines, lexiconPrefix):
		lexcFile.write(replacementsBack(line) + "\n")
		if "<A>" in line:
			lexcFile.write(replacementsFront(line) + "\n")

def appendLexicon(lexiconName, lines, lexcFile):
	if lexiconName.startswith("NOUN "):
		realName = lexiconName[5:]
		appendLines("Nimisana", realName, lines, lexcFile)
		appendLines("Laatusana", realName, lines, lexcFile)
		appendLines("Etunimi", realName, lines, lexcFile)
		appendLines("Sukunimi", realName, lines, lexcFile)
		appendLines("Paikannimi", realName, lines, lexcFile)
		appendLines("Nimi", realName, lines, lexcFile)
		appendLines("NimiLaatusana", realName, lines, lexcFile)
		if realName in ["Vieras", "Vieras_s", "Vieras_w", "YhteisetMonikonPaikallissijat", \
		                "YhteisetYksikönPaikallissijat", "LiOlN", "YhteisetMonikonSijat2", \
		                "LiOlI", "YksikönGenetiivinJatko", "LiOlV", "LiOlAA", "MonikonGenetiiviEnJatko", \
		                "NormaaliYsJatko", "Nainen", "NainenYhteiset", "NainenYsJatko", "Autio", \
		                "NormaaliYsJatkoOl", "Luku", "Valo_w", "Valo_s", "Valo_sl", \
		                "Koira", "Koira_w", "Koira_s", "MonikonGenetiiviInJatko", \
		                "Koira_w_monikko", "Koira_w_yksikkö", "Puu", "Maa_l", "Maa_s", \
		                "OlVLoppu", "MonikonNominatiivinJatko", "YksikönGenetiivinJatkoEiYs"]:
			appendLines("LukusananJälkiliite", realName, lines, lexcFile)
			appendLines("Asemosana", realName, lines, lexcFile)
		if (OPTIONS["sukija"] or OPTIONS["vanhat"]) and realName in ["YhteisetHTaivutusmuodot"]:
			appendLines("LukusananJälkiliite", realName, lines, lexcFile)
			appendLines("Asemosana", realName, lines, lexcFile)
		if OPTIONS["sukija"] and realName in ["Kala_s", "Kala_w", "Kala_w_ei_yliaste", "Pata"]:
			appendLines("Asemosana", realName, lines, lexcFile)
	else:
		appendLines("", lexiconName, lines, lexcFile)

# Get command line options
OPTIONS = generate_lex_common.get_options()

lexcFile = codecs.open(OPTIONS["destdir"] + "/" + "taivutuskaavat.lexc", 'w', 'UTF-8')

infile = codecs.open("vvfst/taivutuskaavat.lexc.in", "r", "UTF-8")

lexicon = ""
lexcLines = []
linecount = 0
while True:
	line_orig = infile.readline()
	linecount = linecount + 1
	if line_orig == '':
		break
	line = generate_lex_common.filterVfstInput(line_orig, OPTIONS)
	if line is None:
		continue
	if line.startswith('LEXICON '):
		if lexicon != "":
			appendLexicon(lexicon, lexcLines, lexcFile)
		lexicon = line[8:]
		lexcLines = []
		continue
	lexcLines.append(line)
infile.close()
	
appendLexicon(lexicon, lexcLines, lexcFile)

# Generate lexicons for numerals

MULTI = {
	"SnNy": ["kymmenen", "sata", "tuhat", "miljoona", "miljardi", "biljoona", "triljoona", "kvadriljoona", "kvintiljoona", "sekstiljoona", "septiljoona", "sentiljoona"],
	"SgNy": ["kymmenen", "sadan", "tuhannen", "miljoonan", "miljardin", "biljoonan", "triljoonan", "kvadriljoonan", "kvintiljoonan", "sekstiljoonan", "septiljoonan", "sentiljoonan"],
	"SpNy": ["kymmentä", "sataa", "tuhatta", "miljoonaa", "miljardia", "biljoonaa", "triljoonaa", "kvadriljoonaa", "kvintiljoonaa", "sekstiljoonaa", "septiljoonaa", "sentiljoonaa"],
	"StrNy": ["kymmeneksi", "sadaksi", "tuhanneksi", "miljoonaksi", "miljardiksi", "biljoonaksi", "triljoonaksi", "kvadriljoonaksi", "kvintiljoonaksi", "sekstiljoonaksi", "septiljoonaksi", "sentiljoonaksi"],
	"SesNy": ["kymmenenä", "satana", "tuhantena", "miljoonana", "miljardina", "biljoonana", "triljoonana", "kvadriljoonana", "kvintiljoonana", "sekstiljoonana", "septiljoonana", "sentiljoonana"],
	"SineNy": ["kymmenessä", "sadassa", "tuhannessa", "miljoonassa", "miljardissa", "biljoonassa", "triljoonassa", "kvadriljoonassa", "kvintiljoonassa", "sekstiljoonassa", "septiljoonassa", "sentiljoonassa"],
	"SelaNy": ["kymmenestä", "sadasta", "tuhannesta", "miljoonasta", "miljardista", "biljoonasta", "triljoonasta", "kvadriljoonasta", "kvintiljoonasta", "sekstiljoonasta", "septiljoonasta", "sentiljoonasta"],
	"SillNy": ["kymmeneen", "sataan", "tuhanteen", "miljoonaan", "miljardiin", "biljoonaan", "triljoonaan", "kvadriljoonaan", "kvintiljoonaan", "sekstiljoonaan", "septiljoonaan", "sentiljoonaan"],
	"SadeNy": ["kymmenellä", "sadalla", "tuhannella", "miljoonalla", "miljardilla", "biljoonalla", "triljoonalla", "kvadriljoonalla", "kvintiljoonalla", "sekstiljoonalla", "septiljoonalla", "sentiljoonalla"],
	"SablNy": ["kymmeneltä", "sadalta", "tuhannelta", "miljoonalta", "miljardilta", "biljoonalta", "triljoonalta", "kvadriljoonalta", "kvintiljoonalta", "sekstiljoonalta", "septiljoonalta", "sentiljoonalta"],
	"SallNy": ["kymmenelle", "sadalle", "tuhannelle", "miljoonalle", "miljardille", "biljoonalle", "triljoonalle", "kvadriljoonalle", "kvintiljoonalle", "sekstiljoonalle", "septiljoonalle", "sentiljoonalle"],
	"SabNy": ["kymmenettä", "sadatta", "tuhannetta", "miljoonatta", "miljarditta", "biljoonatta", "triljoonatta", "kvadriljoonatta", "kvintiljoonatta", "sekstiljoonatta", "septiljoonatta", "sentiljoonatta"],
	"SgNm": ["kymmenien", "satojen", "tuhansien", "miljoonien", "miljardien", "biljoonien", "triljoonien", "kvadriljoonien", "kvintiljoonien", "sekstiljoonien", "septiljoonien", "sentiljoonien"],
	"SpNm": ["kymmeniä", "satoja", "tuhansia", "miljoonia", "miljardeja", "biljoonia", "triljoonia", "kvadriljoonia", "kvintiljoonia", "sekstiljoonia", "septiljoonia", "sentiljoonia"],
	"StrNm": ["kymmeniksi", "sadoiksi", "tuhansiksi", "miljooniksi", "miljardeiksi", "biljooniksi", "triljooniksi", "kvadriljooniksi", "kvintiljooniksi", "sekstiljooniksi", "septiljooniksi", "sentiljooniksi"],
	"SesNm": ["kymmeninä", "satoina", "tuhansina", "miljoonina", "miljardeina", "biljoonina", "triljoonina", "kvadriljoonina", "kvintiljoonina", "sekstiljoonina", "septiljoonina", "sentiljoonina"],
	"SineNm": ["kymmenissä", "sadoissa", "tuhansissa", "miljoonissa", "miljardeissa", "biljoonissa", "triljoonissa", "kvadriljoonissa", "kvintiljoonissa", "sekstiljoonissa", "septiljoonissa", "sentiljoonissa"],
	"SelaNm": ["kymmenistä", "sadoista", "tuhansista", "miljoonista", "miljardeista", "biljoonista", "triljoonista", "kvadriljoonista", "kvintiljoonista", "sekstiljoonista", "septiljoonista", "sentiljoonista"],
	"SillNm": ["kymmeniin", "satoihin", "tuhansiin", "miljooniin", "miljardeihin", "biljooniin", "triljooniin", "kvadriljooniin", "kvintiljooniin", "sekstiljooniin", "septiljooniin", "sentiljooniin"],
	"SadeNm": ["kymmenillä", "sadoilla", "tuhansilla", "miljoonilla", "miljardeilla", "biljoonilla", "triljoonilla", "kvadriljoonilla", "kvintiljoonilla", "sekstiljoonilla", "septiljoonilla", "sentiljoonilla"],
	"SablNm": ["kymmeniltä", "sadoilta", "tuhansilta", "miljoonilta", "miljardeilta", "biljoonilta", "triljoonilta", "kvadriljoonilta", "kvintiljoonilta", "sekstiljoonilta", "septiljoonilta", "sentiljoonilta"],
	"SallNm": ["kymmenille", "sadoille", "tuhansille", "miljoonille", "miljardeille", "biljoonille", "triljoonille", "kvadriljoonille", "kvintiljoonille", "sekstiljoonille", "septiljoonille", "sentiljoonille"],
	"SabNm": ["kymmenittä", "sadoitta", "tuhansitta", "miljoonitta", "miljardeitta", "biljoonitta", "triljoonitta", "kvadriljoonitta", "kvintiljoonitta", "sekstiljoonitta", "septiljoonitta", "sentiljoonitta"],
	"SinNm": ["kymmenin", "sadoin", "tuhansin", "miljoonin", "miljardein", "biljoonin", "triljoonin", "kvadriljoonin", "kvintiljoonin", "sekstiljoonin", "septiljoonin", "sentiljoonin"],
	"SkoNm": ["kymmenine", "satoine", "tuhansine", "miljoonine", "miljardeine", "biljoonine", "triljoonine", "kvadriljoonine", "kvintiljoonine", "sekstiljoonine", "septiljoonine", "sentiljoonine"],
	"SstiNy": ["kymmenesti", "sadasti", "tuhannesti", "miljoonasti", "miljardisti", "biljoonasti", "triljoonasti", "kvadriljoonasti", "kvintiljoonasti", "sekstiljoonasti", "septiljoonasti", "sentiljoonasti"]
}

MULTI_VOWELS = ["_ä", "_a", "_a", "_a", "_a", "_a", "_a", "_a", "_a", "_a", "_a", "_a"]

for sija in MULTI.keys():
	if sija == "SnNy":
		continue
	diacritic = "@U.LS." + sija.upper() + "@"
	lexiconName = "Lukusana" + sija + "29"
	tagName = "[" + sija.replace("N", "][N") + "]"
	
	lexcFile.write("LEXICON " + lexiconName + "Kertoimet\n")
	for i in range(len(MULTI_VOWELS)):
		lexcFile.write(diacritic + "[Xp]" + MULTI["SpNy"][i] + "[X]" + tagName + MULTI[sija][i] + ":" + diacritic + MULTI[sija][i] + "\t" + lexiconName + MULTI_VOWELS[i] + "\t;\n")
		if MULTI[sija][i].endswith("joonien"):
			altSija = MULTI[sija][i].replace("joonien", "joonain")
			lexcFile.write(diacritic + tagName + altSija + ":" + diacritic + altSija + "\t" + lexiconName + MULTI_VOWELS[i] + "\t;\n")
	
	lexcFile.write("LEXICON " + lexiconName + "Alut\n")
	for i in range(len(MULTI_VOWELS)):
		lexcFile.write(diacritic + "[Xp]" + MULTI["SnNy"][i] + "[X]" + tagName + MULTI[sija][i] + ":" + diacritic + MULTI[sija][i] + "\t" + lexiconName + MULTI_VOWELS[i] + "\t;\n")
		if MULTI[sija][i].endswith("joonien"):
			altSija = MULTI[sija][i].replace("joonien", "joonain")
			lexcFile.write(diacritic + tagName + altSija + ":" + diacritic + altSija + "\t" + lexiconName + MULTI_VOWELS[i] + "\t;\n")
	
	numeralLines = []
	numeralLines.append(diacritic + ":" + diacritic + "\tLukusanaLiitesana_<A>\t;")
	numeralLines.append(diacritic + ":" + diacritic + "\tLukusanaToista\t;")
	if OPTIONS["sukija"]:
		numeralLines.append(diacritic + ":" + diacritic + "\tSukijaLukusanaKolmattaYhdeksättä\t;")
	if sija in ["SelaNy", "SelaNm"]:
		numeralLines.append(diacritic + ":" + diacritic + "\tOlV_<A>\t;")
	appendLines("Lukusana", sija + "29", numeralLines, lexcFile)

	numeralLines = []
	numeralLines.append(diacritic + ":" + diacritic + "\tLiitesana_<A>\t;")
	numeralLines.append(diacritic + ":" + diacritic + "\tLukusanaToista\t;")
	if OPTIONS["sukija"]:
		numeralLines.append(diacritic + ":" + diacritic + "\tSukijaLukusanaKolmattaYhdeksättä\t;")
	numeralLines.append(diacritic + ":" + diacritic + "\tLukusananJälkiliite\t;")
	if sija == "SelaNm":
		numeralLines.append(diacritic + ":" + diacritic + "\tOlV_<A>\t;")
	if sija == "StrNm":
		numeralLines.append(diacritic + ":" + diacritic + "\tOlI_<A>\t;")
	appendLines("Lukusana", sija + "1", numeralLines, lexcFile)

MULTI_ORDINAL_BASES = ["kymmenes", "sadas", "tuhannes", "miljoonas", "miljardis"]

MULTI_ORDINALS = {
	"SgNy": ["kymmenennen", "sadannen", "tuhannennen", "miljoonannen", "miljardinnen"],
	"SpNy": ["kymmenettä", "sadannetta", "tuhannetta", "miljoonannetta", "miljardinnetta"],
	"StrNy": ["kymmenenneksi", "sadanneksi", "tuhannenneksi", "miljoonanneksi", "miljardinneksi"],
	"SesNy": ["kymmenentenä", "sadantena", "tuhannentena", "miljoonantena", "miljardintena"],
	"SineNy": ["kymmenennessä", "sadannessa", "tuhannennessa", "miljoonannessa", "miljardinnessa"],
	"SelaNy": ["kymmenennestä", "sadannesta", "tuhannennesta", "miljoonannesta", "miljardinnesta"],
	"SillNy": ["kymmenenteen", "sadanteen", "tuhannenteen", "miljoonanteen", "miljardinteen"],
	"SadeNy": ["kymmenennellä", "sadannella", "tuhannennella", "miljoonannella", "miljardinnella"],
	"SablNy": ["kymmenenneltä", "sadannelta", "tuhannennelta", "miljoonannelta", "miljardinnelta"],
	"SallNy": ["kymmenennelle", "sadannelle", "tuhannennelle", "miljoonannelle", "miljardinnelle"],
	"SabNy": ["kymmenennettä", "sadannetta", "tuhannennetta", "miljoonannetta", "miljardinnetta"],
	"SgNm": ["kymmenensien", "sadansien", "tuhannensien", "miljoonansien", "miljardinsien"],
	"SpNm": ["kymmenensiä", "sadansia", "tuhannensia", "miljoonansia", "miljardinsia"],
	"StrNm": ["kymmenensiksi", "sadansiksi", "tuhannensiksi", "miljoonansiksi", "miljardinsiksi"],
	"SesNm": ["kymmenensinä", "sadansina", "tuhannensina", "miljoonansina", "miljardinsina"],
	"SineNm": ["kymmenensissä", "sadansissa", "tuhannensissa", "miljoonansissa", "miljardinsissa"],
	"SelaNm": ["kymmenensistä", "sadansista", "tuhannensista", "miljoonansista", "miljardinsista"],
	"SillNm": ["kymmenensiin", "sadansiin", "tuhannensiin", "miljoonansiin", "miljardinsiin"],
	"SadeNm": ["kymmenensillä", "sadansilla", "tuhannensilla", "miljoonansilla", "miljardinsilla"],
	"SablNm": ["kymmenensiltä", "sadansilta", "tuhannensilta", "miljoonansilta", "miljardinsilta"],
	"SallNm": ["kymmenensille", "sadansille", "tuhannensille", "miljoonansille", "miljardinsille"],
	"SabNm": ["kymmenensittä", "sadansitta", "tuhannensitta", "miljoonansitta", "miljardinsitta"],
	"SinNm": ["kymmenensin", "sadansin", "tuhannensin", "miljoonansin", "miljardinsin"],
	"SkoNm": ["kymmenensine", "sadansine", "tuhannensine", "miljoonansine", "miljardinsine"]
}

for sija in MULTI_ORDINALS.keys():
	diacritic = "@U.LS." + sija.upper() + "@"
	lexiconName = "Järjestysluku" + sija + "39"
	tagName = "[" + sija.replace("N", "][N") + "]"
	
	lexcFile.write("LEXICON " + lexiconName + "Kertoimet\n")
	lexcFile.write("[Bc]" + diacritic + "[Xp]" + MULTI_ORDINAL_BASES[0] + "[X]" + tagName + MULTI_ORDINALS[sija][0] + ":" + diacritic + MULTI_ORDINALS[sija][0] + "\t" + lexiconName + "_ä\t;\n")
	lexcFile.write("[Bc]" + diacritic + "[Xp]" + MULTI_ORDINAL_BASES[1] + "[X]" + tagName + MULTI_ORDINALS[sija][1] + ":" + diacritic + MULTI_ORDINALS[sija][1] + "\t" + lexiconName + "_a\t;\n")
	lexcFile.write("[Bc]" + diacritic + "[Xp]" + MULTI_ORDINAL_BASES[2] + "[X]" + tagName + MULTI_ORDINALS[sija][2] + ":" + diacritic + MULTI_ORDINALS[sija][2] + "\t" + lexiconName + "_a\t;\n")
	lexcFile.write("[Bc]" + diacritic + "[Xp]" + MULTI_ORDINAL_BASES[3] + "[X]" + tagName + MULTI_ORDINALS[sija][3] + ":" + diacritic + MULTI_ORDINALS[sija][3] + "\t" + lexiconName + "_a\t;\n")
	lexcFile.write("[Bc]" + diacritic + "[Xp]" + MULTI_ORDINAL_BASES[4] + "[X]" + tagName + MULTI_ORDINALS[sija][4] + ":" + diacritic + MULTI_ORDINALS[sija][4] + "\t" + lexiconName + "_a\t;\n")
	
	numeralLines = []
	if sija in ["SgNy", "SgNm"]:
		numeralLines.append(diacritic + ":" + diacritic + "\tJärjestyslukuLiitesanaJl_<A>\t;")
	else:
		numeralLines.append(diacritic + ":" + diacritic + "\tJärjestyslukuLiitesana_<A>\t;")
	numeralLines.append(diacritic + ":" + diacritic + "\tJärjestyslukuToista\t;")
	if OPTIONS["sukija"]:
		numeralLines.append(diacritic + ":" + diacritic + "\tSukijaJärjestyslukuKolmattaYhdeksättä\t;")
	numeralLines.append(lexiconName + "Kertoimet\t;")
	appendLines("Järjestysluku", sija + "39", numeralLines, lexcFile)
	
	numeralLines = []
	numeralLines.append(diacritic + ":" + diacritic + "\tLiitesana_<A>\t;")
	numeralLines.append(diacritic + ":" + diacritic + "\tJärjestyslukuToista\t;")
	if OPTIONS["sukija"]:
		numeralLines.append(diacritic + ":" + diacritic + "\tSukijaJärjestyslukuKolmattaYhdeksättä\t;")
	numeralLines.append(diacritic + ":" + diacritic + "\tLukusananJälkiliiteJl\t;")
	appendLines("Järjestysluku", sija + "1", numeralLines, lexcFile)


lexcFile.close()



# Copyright 2007 - 2015 Harri Pitkänen (hatapitk@iki.fi)
# Program to generate lexicon files for VFST variant of voikko-fi

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
sys.path.append("common")
import hfconv
import generate_lex_common
import voikkoutils
import xml.dom.minidom
import codecs
from xml.dom import Node

flag_attributes = voikkoutils.readFlagAttributes(generate_lex_common.VOCABULARY_DATA + "/flags.txt")

# Get command line options
OPTIONS = generate_lex_common.get_options()

# Inflection class map
CLASSMAP = hfconv.compileClassmapREs(hfconv.modern_classmap)

vocabularyFileSuffixes = ["ep", "ee", "es", "em", "t", "nl", "l", "n", "h", "p", "a", "s", "c"]
vocabularyFiles = {}
for fileSuffix in vocabularyFileSuffixes:
	vocFile = codecs.open(OPTIONS["destdir"] + "/joukahainen-" + fileSuffix + ".lexc", 'w', 'UTF-8')
	vocFile.write("! This is automatically generated intermediate lexicon file for\n")
	vocFile.write("! VVFST morphology of voikko-fi. The original source data is\n")
	vocFile.write("! distributed under the GNU General Public License, version 2 or\n")
	vocFile.write("! later, as published by the Free Software Foundation. You should\n")
	vocFile.write("! have received the original data, tools and instructions to\n")
	vocFile.write("! generate this file (or instructions to obtain them) wherever\n")
	vocFile.write("! you got this file from.\n\n")
	vocFile.write("LEXICON Joukahainen_" + fileSuffix + "\n")
	vocabularyFiles[fileSuffix] = vocFile


def frequency(word):
	fclass = word.getElementsByTagName("fclass")
	if len(fclass) == 0: return 7
	return int(generate_lex_common.tValue(fclass[0]))

# Check the style flags of the word according to current options.
# Returns True if the word is acceptable, otherwise returns false.
def check_style(word):
	global OPTIONS
	for styleE in word.getElementsByTagName("style"):
		for style in generate_lex_common.tValues(styleE, "flag"):
			if style == "foreignloan":
				continue
			if not style in OPTIONS["style"]: return False
	return True

# Returns True if the word is acceptable according to its usage flags.
def check_usage(word):
	global OPTIONS
	wordUsage = word.getElementsByTagName("usage")
	if len(wordUsage) == 0: return True
	for usageE in wordUsage:
		for usage in generate_lex_common.tValues(usageE, "flag"):
			if usage in OPTIONS["extra-usage"]: return True
	return False

# Returns VFST word class for given word in Joukahainen
def get_vfst_word_class(j_wordclasses):
	if "pnoun_place" in j_wordclasses: return "[Lep]"
	if "pnoun_firstname" in j_wordclasses: return "[Lee]"
	if "pnoun_lastname" in j_wordclasses: return "[Les]"
	if "pnoun_misc" in j_wordclasses: return "[Lem]"
	if "verb" in j_wordclasses: return "[Lt]"
	if "adjective" in j_wordclasses and "noun" in j_wordclasses: return "[Lnl]"
	if "adjective" in j_wordclasses: return "[Ll]"
	if "noun" in j_wordclasses: return "[Ln]"
	if "interjection" in j_wordclasses: return "[Lh]"
	if "prefix" in j_wordclasses: return "[Lp]"
	if "abbreviation" in j_wordclasses: return "[La]"
	if "adverb" in j_wordclasses: return "[Ls]"
	if "conjunction" in j_wordclasses: return "[Lc]"
	return None

# Returns a string describing the structure of a word, if necessary for the spellchecker
# or hyphenator
def get_structure(wordform, vfst_word_class, alku):
	needstructure = False
	ispropernoun = vfst_word_class[0:3] == '[Le'
	structstr = '[Xr]'
	oldAlku = alku
	newAlku = ""
	if vfst_word_class == '[La]':
		i = "j"
		p = "q"
	else:
		i = "i"
		p = "p"
	for idx in range(len(wordform)):
		c = wordform[idx]
		if c == '-':
			structstr = structstr + "-="
			if (len(oldAlku) > 0):
				newAlku = newAlku + '-[Bm]'
				oldAlku = oldAlku[1:]
		elif c == '|':
			structstr = structstr
		elif c == '=':
			structstr = structstr + "="
			newAlku = newAlku + "[Bm]"
		elif c == ':':
			structstr = structstr + ":"
			needstructure = True
			if (len(oldAlku) > 0):
				newAlku = newAlku + ':'
				oldAlku = oldAlku[1:]
		elif c.isupper():
			structstr = structstr + i
			if not (ispropernoun and idx == 0):
				needstructure = True
			if (len(oldAlku) > 0):
				newAlku = newAlku + oldAlku[0]
				oldAlku = oldAlku[1:]
		else:
			structstr = structstr + p
			if ispropernoun and idx == 0:
				needstructure = True
			if (len(oldAlku) > 0):
				newAlku = newAlku + oldAlku[0]
				oldAlku = oldAlku[1:]
	if needstructure:
		returnedLength = len(structstr)
		while structstr[returnedLength - 1] == p:
			returnedLength = returnedLength - 1
		return (structstr[0:returnedLength] + '[X]', alku)
	else:
		return ("", newAlku)

def get_diacritics(word, altforms, vfst_word_class):
	diacritics = []
	for group in word.childNodes:
		if group.nodeType != Node.ELEMENT_NODE:
			continue
		for flag in group.childNodes:
			if flag.nodeType != Node.ELEMENT_NODE:
				continue
			if flag.tagName != "flag":
				continue
			flagName = flag.firstChild.wholeText
			if flagName == "ei_yks":
				diacritics.append("@P.EI_YKS.ON@")
			elif flagName == "ysj":
				diacritics.append("@R.YS_ALKANUT@")
			elif flagName == "inen":
				diacritics.append("@P.INEN_SALLITTU.ON@")
			elif flagName == "ei_inen":
				diacritics.append("@P.INEN_KIELLETTY.ON@")
			elif flagName == "ei_mainen":
				diacritics.append("@P.EI_MAINEN.ON@")
			elif flagName == "ei_lainen":
				diacritics.append("@P.EI_LAINEN.ON@")
			elif flagName == "ei_vertm":
				diacritics.append("@P.EI_VERTM.ON@")
			elif flagName == "ym3":
				diacritics.append("@P.VAIN_YM3.ON@")
			elif flagName == "yt":
				diacritics.append("@P.YKSITEKIJÄINEN.ON@")
			elif flagName == "geo_suffix":
				diacritics.append("@C.PAIKANNIMEN_JL@")
			if flagName in ["ei_ys", "ei_ysa"]:
				diacritics.append("@P.YS_EI_JATKOA.ON@")
			if flagName in ["ei_ys", "ei_ysj"]:
				diacritics.append("@D.YS_ALKANUT@")
	if vfst_word_class in ["[Ln]", "[Lnl]"] and (altforms[0].endswith("lainen") or altforms[0].endswith("läinen")):
		diacritics.append("@P.LAINEN.ON@@C.LAINEN_VAADITTU@@C.VAIN_NIMISANA@")
	return diacritics

def get_info_flags(word):
	flags = ""
	for group in word.childNodes:
		if group.nodeType != Node.ELEMENT_NODE:
			continue
		for flag in group.childNodes:
			if flag.nodeType != Node.ELEMENT_NODE:
				continue
			if flag.tagName != "flag":
				continue
			flagName = flag.firstChild.wholeText
			if flagName == "paikannimi_ulkopaikallissijat":
				flags = flags + "[Ipu]"
			elif flagName == "paikannimi_sisäpaikallissijat":
				flags = flags + "[Ips]"
			elif flagName == "foreignloan":
				flags = flags + "[Isf]"
			elif flagName == "el_altark":
				flags = flags + "[De]"
			elif flagName == "geo_suffix":
				flags = flags + "[Ica]"
			elif flagName == "org_suffix":
				flags = flags + "[Ion]"
			elif flagName == "free_suffix":
				flags = flags + "[Ivj]"
			elif flagName == "require_following_a":
				flags = flags + "[Ira]"
			elif flagName == "require_following_ma":
				flags = flags + "[Irm]"
	return flags

def get_vfst_class_prefix(vfst_class):
	if vfst_class == "[Ln]":
		return "Nimisana"
	elif vfst_class == "[Lee]":
		return "Etunimi"
	elif vfst_class == "[Lep]":
		return "Paikannimi"
	elif vfst_class == "[Les]":
		return "Sukunimi"
	elif vfst_class == "[Lem]":
		return "Nimi"
	elif vfst_class == "[Ll]":
		return "Laatusana"
	elif vfst_class == "[Lnl]":
		return "NimiLaatusana"
	else:
		return ""

def vowel_type_for_derived_verb(wordform):
	for char in reversed(wordform):
		if char in "yäö":
			return "@P.V_SALLITTU.E@"
		if char in "uao":
			return "@P.V_SALLITTU.T@"
		if char in "]":
			break
	return "@P.V_SALLITTU.T@"

def get_prefix_jatko(word, altform):
	flags = generate_lex_common.get_flags_from_group(word, "compounding")
	prefixJatko = ""
	for flag in sorted(flags):
		if flag in ["eln", "ell", "elt", "eltj"]:
			prefixJatko = prefixJatko + flag
	if altform.endswith("-"):
		prefixJatko = prefixJatko + "H"
	return prefixJatko

def get_adverb_jatko(word, altform):
	flags = generate_lex_common.get_flags_from_group(word, "inflection")
	loppu = True
	adverbJatko = ""
	for flag in sorted(flags):
		if flag in ["liitesana", "ulkopaikallissijat_yks"]:
			adverbJatko = adverbJatko + flag.title()
		elif flag == "omistusliite":
			if altform[-1] in "aäe" and altform[-1] != altform[-2]:
				adverbJatko = adverbJatko + "OlV"
			else:
				adverbJatko = adverbJatko + "Omistusliite"
		elif flag == "required":
			loppu = False;
	if loppu:
		adverbJatko = "Loppu" + adverbJatko
	return adverbJatko

def get_abbreviation_jatko(word, wordform):
	flags = generate_lex_common.get_flags_from_group(word, "inflection")
	if wordform.endswith(".") or "none" in flags:
		return "PisteellisenLyhenteenJatko"
	else:
		return "Lyhenne"

def injectBaseformToStructure(baseform, structure):
	if baseform is None or baseform == "":
		return structure
	i = 0
	# The following requires an incompatible format change but would
	# make the standard dictionary about 30 % smaller.
	# while i < len(baseform) and i + 1 < len(structure):
	#	if baseform[i] != structure[i]:
	#		break
	#	i = i + 1
	if i == 0:
		return "[Xp]" + baseform + "[X]" + structure
	return structure[:i] + "[Xp]" + baseform[i:] + "[X]" + structure[i:]

def handle_word(word):
	global OPTIONS
	global CLASSMAP
	# Drop words that are not needed in the Voikko lexicon
	# but only if not generating Sukija lexicon.
	if generate_lex_common.has_flag(word, "not_voikko") and not OPTIONS["sukija"]: return
	if not check_style(word): return
	if not check_usage(word): return
	if frequency(word) >= OPTIONS["frequency"] + 1: return
	if frequency(word) == OPTIONS["frequency"] and generate_lex_common.has_flag(word, "confusing"): return
	
	# Get the inflection class. Exactly one inflection class is needed
	voikko_infclass = None
	if OPTIONS["sukija"]:
		for infclass in word.getElementsByTagName("infclass"):
			if infclass.getAttribute("type") == "historical":
				voikko_infclass = generate_lex_common.tValue(infclass)
				if voikko_infclass == "banaali":   # Banaali taipuu kuten paperi.
					voikko_infclass = "paperi"
				elif voikko_infclass == "pasuuna": # Pasuuna tapuu kuten peruna.
					voikko_infclass = "peruna"
#				if voikko_infclass in ["ahven", "altis", "bébé", "haastaa", "herttua", "hyljätä", "hylätä",
#                                                       "jälki", "kaihtaa", "kalsium", "katketa-av4",
##                                                       "kantaja",
#                                                       "kirjoitella", "kulkija", "kutiaa", "laittaa", "lampi", "levätä",
#                                                       "nuori", "paahtaa", "paistaa", "palaa", "palata", "peitsi",
#                                                       "siivota", "sydän", "taittaa", "terve", "tulla", "tuomi", "uros",
#                                                       "valmis", "veranta", "virkkaa"]:
#					voikko_infclass = None
#
				if voikko_infclass not in ["aavistaa-av1", "antautua", "arvelu", "arvelu-av1", "asema", "asema-av1", "askel", "autio",
                                                           "edam", "haastaa", "haravoida", "haravoida-av2", "heittää-av1", "huuli",
                                                           "kantaja", "karahka", "karahka-av1",
                                                           "kiiski", "koiras", "matala", "paperi", "paperi-av1", "peruna"]:
					voikko_infclass = None
				break
	if voikko_infclass == None:
		for infclass in word.getElementsByTagName("infclass"):
			if infclass.getAttribute("type") != "historical":
				voikko_infclass = generate_lex_common.tValue(infclass)
				break
	if voikko_infclass == "poikkeava": return
	
	# Get the word classes
	wordclasses = generate_lex_common.tValues(word.getElementsByTagName("classes")[0], "wclass")
	if wordclasses[0] not in ["interjection", "prefix", "abbreviation", "conjunction", "adverb"] and voikko_infclass == None:
		return
	vfst_word_class = get_vfst_word_class(wordclasses)
	if vfst_word_class == None: return
	
	# Get diacritics
	altforms = generate_lex_common.tValues(word.getElementsByTagName("forms")[0], "form")
	diacritics = "".join(get_diacritics(word, altforms, vfst_word_class))
	
	# Get forced vowel type
	if voikko_infclass == None and vfst_word_class != "[La]":
		forced_inflection_vtype = voikkoutils.VOWEL_DEFAULT
	else:
		inflectionElement = word.getElementsByTagName("inflection")
		if len(inflectionElement) > 0:
			forced_inflection_vtype = generate_lex_common.vowel_type(inflectionElement[0])
		else:
			forced_inflection_vtype = voikkoutils.VOWEL_DEFAULT
	
	# Construct debug information
	debug_info = ""
	if OPTIONS["sourceid"]:
		debug_info = '[Xs]%s[X]' % word.getAttribute("id")[1:].replace("0", "%0")
	
	infoFlags = get_info_flags(word)
	
	# Process all alternative forms
	singlePartForms = []
	multiPartForms = []
	for altform in altforms:
		outputBaseform = altform.replace('|', '')
		wordform = outputBaseform.replace('=', '')
		if len(altform) == len(wordform.replace('-', '')):
			singlePartForms.append(altform)
		else:
			multiPartForms.append(altform)
		(alku, jatko) = generate_lex_common.get_malaga_inflection_class(wordform, voikko_infclass, wordclasses, CLASSMAP)
		if alku == None:
			errorstr = "ERROR: VFST class not found for (%s, %s)\n" % (wordform, voikko_infclass)
			sys.stderr.write(errorstr)
			sys.exit(1)
		if vfst_word_class == "[La]":
			jatko = get_abbreviation_jatko(word, altform)
		elif vfst_word_class == "[Ls]":
			jatko = get_adverb_jatko(word, altform)
		else:
			jatko = jatko.title()
		if vfst_word_class in ["[Ls]", "[Lc]", "[Lh]"]:
			for element in word.getElementsByTagName("baseform"):
				wordform = generate_lex_common.tValue(element)
				outputBaseform = wordform.replace('|', '')
		if forced_inflection_vtype == voikkoutils.VOWEL_DEFAULT:
			vtype = voikkoutils.get_wordform_infl_vowel_type(altform)
		else: vtype = forced_inflection_vtype
		if vtype == voikkoutils.VOWEL_FRONT: vfst_vtype = 'ä'
		elif vtype == voikkoutils.VOWEL_BACK: vfst_vtype = 'a'
		elif vtype == voikkoutils.VOWEL_BOTH: vfst_vtype = 'aä'
		vocabularyFile = vocabularyFiles[vfst_word_class.replace("[L", "").replace("]", "")]
		if alku == None:
			errorstr = "ERROR: Malaga class not found for (%s, %s)\n" \
				% (wordform, voikko_infclass)
			generate_lex_common.write_entry(vocabularyFile, {}, word, errorstr)
			sys.stderr.write(errorstr)
			sys.exit(1)
		alku = alku.lower()
		(rakenne, alkuWithTags) = get_structure(altform, vfst_word_class, alku)
		
		if OPTIONS["no-baseform"]:
			outputBaseform = ""
		
		if vfst_word_class == "[Lh]":
			entry = '%s%s%s%s:%s # ;' % (vfst_word_class, debug_info, rakenne, injectBaseformToStructure(outputBaseform, alkuWithTags), alku)
			vocabularyFile.write(entry + "\n")
			continue
		vfst_class_prefix = get_vfst_class_prefix(vfst_word_class)
		
		# Vowel type in derived verbs
		if jatko in ["Heittää", "Muistaa", "Juontaa", "Hohtaa", "Murtaa", "Nousta", "Loistaa", "Jättää", "Kihistä", "Kyntää2"]:
			diacritics = diacritics + vowel_type_for_derived_verb(alkuWithTags)
			if jatko == "Kihistä" and vtype == voikkoutils.VOWEL_FRONT and "y" not in alku and "ä" not in alku and "ö" not in alku and "e" in alku:
				jatko = "Helistä"
		
		if jatko == "Nainen" and vfst_class_prefix in ["Laatusana", "NimiLaatusana"] and altform.endswith("inen"):
			jatko = "NainenInen"
		
		if vfst_word_class == "[Lp]":
			entry = '[Lp]%s%s%s%s%s:%s%s EtuliitteenJatko_%s;' \
			        % (debug_info, rakenne, alkuWithTags, diacritics, infoFlags, alku, diacritics, get_prefix_jatko(word, altform))
		else:
			entry = '%s%s%s%s%s%s:%s%s %s%s_%s ;' \
			        % (vfst_word_class, debug_info, rakenne, infoFlags, injectBaseformToStructure(outputBaseform, alkuWithTags),
			        diacritics, alku, diacritics, vfst_class_prefix, jatko, vfst_vtype)
		vocabularyFile.write(entry + "\n")
	
	# Sanity check for alternative forms: if there are both multi part forms and single part forms
	# then all multi part forms must end with a part contained in the single part set.
	if singlePartForms:
		for multiPartForm in multiPartForms:
			lastPart = multiPartForm[max(multiPartForm.rfind("="), multiPartForm.rfind("|"), multiPartForm.rfind("-")) + 1:]
			if lastPart not in singlePartForms:
				sys.stderr.write("ERROR: suspicious alternative spelling: %s\n" % multiPartForm)
				sys.exit(1)


voikkoutils.process_wordlist(generate_lex_common.VOCABULARY_DATA + '/joukahainen.xml', \
                             handle_word, True)

for fileSuffix in vocabularyFileSuffixes:
	vocabularyFiles[fileSuffix].write("\n\n") # Extra line feeds needed to avoid mixed lines in concatenated lexc file
	vocabularyFiles[fileSuffix].close()



# Copyright 2015 Harri Pitkänen (hatapitk@iki.fi)
# Program for removing comments and optionally disabled entries from
# LEXC files.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
sys.path.append("common")
import codecs
import fileinput
import generate_lex_common

outputName = sys.argv[1]
files = [f for f in sys.argv[2:] if not f.startswith("--")]

OPTIONS = generate_lex_common.get_options()
outputFile = codecs.open(outputName, 'w', 'UTF-8')

for line_orig in fileinput.input(files,openhook=fileinput.hook_encoded("UTF-8")):
	line = generate_lex_common.filterVfstInput(line_orig, OPTIONS)
	if line is None:
		continue
	outputFile.write(line + "\n")



# Copyright 2009 - 2015 Harri Pitkänen (hatapitk@iki.fi)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# This program converts an XML representation of autocorrect data
# into lexc format for autocorrect transducer.
#
# Usage: python triecompiler.py input.xml output.lexc

import xml.dom.minidom
import sys

# Open the XML file
xmlFile = open(sys.argv[1], "r", encoding="UTF-8")
autoCorrect = xml.dom.minidom.parseString(xmlFile.read())
xmlFile.close()

# Open the lexc file
outputFile = open(sys.argv[2], "w", encoding="UTF-8")
outputFile.write("Multichar_Symbols\n@_SPACE_@\n\nLEXICON Root\n")

def formatForLexc(s):
	return s.replace("=", "").replace(" ", "@_SPACE_@")

# Read entries to lexc
for replacement in autoCorrect.getElementsByTagName("replacement"):
	incorrect = formatForLexc(replacement.getElementsByTagName("incorrect")[0].firstChild.wholeText)
	correct = formatForLexc(replacement.getElementsByTagName("correct")[0].firstChild.wholeText)
	outputFile.write(incorrect + ":" + correct + "\t#\t;\n")

outputFile.close()



# -*- coding: utf-8 -*-

# Copyright 2007 - 2011 Harri Pitkänen (hatapitk@iki.fi)
#           2007        Hannu Väisänen (Etunimi.Sukunimi@joensuu.fi)
#
# Functions and variables that are common to Sukija and Voikko versions.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import hfconv
import voikkoutils
import codecs
import getopt
import sys
from xml.dom import Node

# Path to source data directory
VOCABULARY_DATA = "vocabulary"

def tValue(element):
	return element.firstChild.wholeText

# Returns a list of text values with given element name under DOM element "group"
def tValues(group, element_name):
	values = []
	for element in group.getElementsByTagName(element_name):
		values.append(tValue(element))
	return values

# Returns flag names from given group for word in Joukahainen
def get_flags_from_group(word, groupName):
	flags = []
	for group in word.childNodes:
		if group.nodeType != Node.ELEMENT_NODE or group.tagName != groupName:
			continue
		for flag in group.childNodes:
			if flag.nodeType != Node.ELEMENT_NODE:
				continue
			if flag.tagName != "flag":
				continue
			flags.append(flag.firstChild.wholeText)
	return flags

flag_attributes = voikkoutils.readFlagAttributes(VOCABULARY_DATA + "/flags.txt")

def vowel_type(group):
	vtypes = group.getElementsByTagName("vtype")
	if len(vtypes) != 1: return voikkoutils.VOWEL_DEFAULT
	else:
		vtypes = tValue(vtypes[0])
		if vtypes == 'a': return voikkoutils.VOWEL_BACK
		elif vtypes == 'ä': return voikkoutils.VOWEL_FRONT
		else: return voikkoutils.VOWEL_BOTH

def has_flag(word, flag):
	if flag in tValues(word, "flag"): return True
	return False

# Returns tuple (alku, jatko) for given word in Joukahainen
def get_malaga_inflection_class(wordform, j_infclass, j_wordclasses, j_classmap):
	if j_infclass is None:
		return (wordform, "loppu")
	(infclass, gradclass) = (list(j_infclass.split('-')) + [None])[:2]
	
	if gradclass == None: gradtypes = [None]
	else: gradtypes = [grad[1] for grad in hfconv.grads if grad[2] == gradclass]
	
	# Determine the word class for the given word
	if "adjective" in j_wordclasses: wclass = hfconv.ADJ
	elif "noun" in j_wordclasses or "pnoun_firstname" in j_wordclasses or \
	     "pnoun_lastname" in j_wordclasses or "pnoun_place" in j_wordclasses or \
	     "pnoun_misc" in j_wordclasses: wclass = hfconv.SUBST
	elif "verb" in j_wordclasses: wclass = hfconv.VERB 
	else: return (None, None)
	
	for (m_infclass, m_infclass_gradation, m_smclasses) in j_classmap:
		if m_infclass != infclass: continue
		for m_smclass in m_smclasses:
			(m_gradtype, pattern, jatko, wclasses) = (list(m_smclass) + [None])[:4]
			if wclasses != None and not wclass in wclasses: continue
			if not m_gradtype in gradtypes: continue
			alku = hfconv.match_re(wordform, pattern)
			if alku != None: return (alku, jatko)
	
	return (None, None)


# Writes the vocabulary entry to a suitable file
def write_entry(main_vocabulary,vocabulary_files,word, entry):
	main_vocabulary.write(entry + "\n")

# Parse command line options and return them in a dictionary
def get_options():
	try:
		optlist = ["min-frequency=", "extra-usage=", "style=", "destdir=", "no-baseform", "sourceid", "vanhat", "sukija", "sukija-ys"]
		(opts, args) = getopt.getopt([f for f in sys.argv[1:] if f.startswith("--")], "", optlist)
	except getopt.GetoptError:
		sys.stderr.write("Invalid option list for %s\n" % sys.argv[0])
		sys.exit(1)
	options = {"frequency": 9,
	           "extra-usage": [],
	           "style": ["old", "international", "inappropriate"],
	           "sourceid": False,
	           "vanhat": False,
	           "destdir": None,
	           "no-baseform": False,
		   "sukija": False,
		   "sukija-ys": False}
	for (name, value) in opts:
		if name == "--min-frequency":
			options["frequency"] = int(value)
		elif name == "--extra-usage":
			options["extra-usage"] = value.split(",")
		elif name == "--style":
			options["style"] = value.split(",")
		elif name == "--destdir":
			options["destdir"] = value
		elif name == "--no-baseform":
			options["no-baseform"] = True
		elif name == "--sourceid":
			options["sourceid"] = True
		elif name == "--vanhat":
			options["vanhat"] = True
		elif name == "--sukija":
			options["sukija"] = True
		elif name == "--sukija-ys":
			options["sukija-ys"] = True
	return options

# Strip whitespace and comments from LEXC input file
def stripWhitespaceAndComments(line):
	if "!" in line:
		line = line[0:line.find("!")]
	return line.strip()

# Filter LEXC input according to options
def filterVfstInput(line_orig, OPTIONS):
	if line_orig.startswith('?Sukija'):
		if OPTIONS["sukija"]:
			line_orig = line_orig[7:]
		else:
			return None
	if line_orig.startswith('?Murre'):
		if "dialect" in OPTIONS["style"] or OPTIONS["sukija"]:
			line_orig = line_orig[6:]
		else:
			return None
	if line_orig.startswith('?Vanha'):
		if OPTIONS["vanhat"] or OPTIONS["sukija"]:
			line_orig = line_orig[6:]
		else:
			return None
	return stripWhitespaceAndComments(line_orig)
